/*
 * WSO2 API Manager - Publisher API
 * This document specifies a **RESTful API** for WSO2 **API Manager** - **Publisher**.  # Authentication The Publisher REST API is protected using OAuth2 and access control is achieved through scopes. Before you start invoking the the API you need to obtain an access token with the required scopes. This guide will walk you through the steps that you will need to follow to obtain an access token. First you need to obtain the consumer key/secret key pair by calling the dynamic client registration (DCR) endpoint. You can add your preferred grant types in the payload. A Sample payload is shown below. ```   {   \"callbackUrl\":\"www.google.lk\",   \"clientName\":\"rest_api_publisher\",   \"owner\":\"admin\",   \"grantType\":\"client_credentials password refresh_token\",   \"saasApp\":true   } ``` Create a file (payload.json) with the above sample payload, and use the cURL shown bellow to invoke the DCR endpoint. Authorization header of this should contain the base64 encoded admin username and password. **Format of the request** ```   curl -X POST -H \"Authorization: Basic Base64(admin_username:admin_password)\" -H \"Content-Type: application/json\"   \\ -d @payload.json https://<host>:<servlet_port>/client-registration/v0.17/register ``` **Sample request** ```   curl -X POST -H \"Authorization: Basic YWRtaW46YWRtaW4=\" -H \"Content-Type: application/json\"   \\ -d @payload.json https://localhost:9443/client-registration/v0.17/register ``` Following is a sample response after invoking the above curl. ``` { \"clientId\": \"fOCi4vNJ59PpHucC2CAYfYuADdMa\", \"clientName\": \"rest_api_publisher\", \"callBackURL\": \"www.google.lk\", \"clientSecret\": \"a4FwHlq0iCIKVs2MPIIDnepZnYMa\", \"isSaasApplication\": true, \"appOwner\": \"admin\", \"jsonString\": \"{\\\"grant_types\\\":\\\"client_credentials password refresh_token\\\",\\\"redirect_uris\\\":\\\"www.google.lk\\\",\\\"client_name\\\":\\\"rest_api123\\\"}\", \"jsonAppAttribute\": \"{}\", \"tokenType\": null } ``` Next you must use the above client id and secret to obtain the access token. We will be using the password grant type for this, you can use any grant type you desire. You also need to add the proper **scope** when getting the access token. All possible scopes for publisher REST API can be viewed in **OAuth2 Security** section of this document and scope for each resource is given in **authorization** section of resource documentation. Following is the format of the request if you are using the password grant type. ``` curl -k -d \"grant_type=password&username=<admin_username>&password=<admin_passowrd&scope=<scopes seperated by space>\" \\ -H \"Authorization: Basic base64(cliet_id:client_secret)\" \\ https://<host>:<servlet_port>/oauth2/token ``` **Sample request** ``` curl https://localhost:9443/oauth2/token -k \\ -H \"Authorization: Basic Zk9DaTR2Tko1OVBwSHVjQzJDQVlmWXVBRGRNYTphNEZ3SGxxMGlDSUtWczJNUElJRG5lcFpuWU1h\" \\ -d \"grant_type=password&username=admin&password=admin&scope=apim:api_view apim:api_create\" ``` Shown below is a sample response to the above request. ``` { \"access_token\": \"e79bda48-3406-3178-acce-f6e4dbdcbb12\", \"refresh_token\": \"a757795d-e69f-38b8-bd85-9aded677a97c\", \"scope\": \"apim:api_create apim:api_view\", \"token_type\": \"Bearer\", \"expires_in\": 3600 } ``` Now you have a valid access token, which you can use to invoke an API. Navigate through the API descriptions to find the required API, obtain an access token as described above and invoke the API with the authentication header. If you use a different authentication mechanism, this process may change.  # Try out in Postman If you want to try-out the embedded postman collection with \"Run in Postman\" option, please follow the guidelines listed below. * All of the OAuth2 secured endpoints have been configured with an Authorization Bearer header with a parameterized access token. Before invoking any REST API resource make sure you run the `Register DCR Application` and `Generate Access Token` requests to fetch an access token with all required scopes. * Make sure you have an API Manager instance up and running. * Update the `basepath` parameter to match the hostname and port of the APIM instance.  [![Run in Postman](https://run.pstmn.io/button.svg)](https://app.getpostman.com/run-collection/a09044034b5c3c1b01a9) 
 *
 * The version of the OpenAPI document: v2
 * Contact: architecture@wso2.com
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */


package org.wso2.am.integration.clients.publisher.api.v1.dto;

import java.util.Objects;
import java.util.Arrays;
import com.google.gson.TypeAdapter;
import com.google.gson.annotations.JsonAdapter;
import com.google.gson.annotations.SerializedName;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.IOException;
import com.fasterxml.jackson.annotation.JsonCreator;
/**
* WebsubSubscriptionConfigurationDTO
*/

public class WebsubSubscriptionConfigurationDTO {
        public static final String SERIALIZED_NAME_ENABLE = "enable";
        @SerializedName(SERIALIZED_NAME_ENABLE)
            private Boolean enable = false;

        public static final String SERIALIZED_NAME_SECRET = "secret";
        @SerializedName(SERIALIZED_NAME_SECRET)
            private String secret;

        public static final String SERIALIZED_NAME_SIGNING_ALGORITHM = "signingAlgorithm";
        @SerializedName(SERIALIZED_NAME_SIGNING_ALGORITHM)
            private String signingAlgorithm;

        public static final String SERIALIZED_NAME_SIGNATURE_HEADER = "signatureHeader";
        @SerializedName(SERIALIZED_NAME_SIGNATURE_HEADER)
            private String signatureHeader;


        public WebsubSubscriptionConfigurationDTO enable(Boolean enable) {
        
        this.enable = enable;
        return this;
        }

    /**
        * Toggle enable WebSub subscription configuration
    * @return enable
    **/
        @javax.annotation.Nullable
      @ApiModelProperty(value = "Toggle enable WebSub subscription configuration")
    
    public Boolean isEnable() {
        return enable;
    }


    public void setEnable(Boolean enable) {
        this.enable = enable;
    }


        public WebsubSubscriptionConfigurationDTO secret(String secret) {
        
        this.secret = secret;
        return this;
        }

    /**
        * Secret key to be used for subscription
    * @return secret
    **/
        @javax.annotation.Nullable
      @ApiModelProperty(value = "Secret key to be used for subscription")
    
    public String getSecret() {
        return secret;
    }


    public void setSecret(String secret) {
        this.secret = secret;
    }


        public WebsubSubscriptionConfigurationDTO signingAlgorithm(String signingAlgorithm) {
        
        this.signingAlgorithm = signingAlgorithm;
        return this;
        }

    /**
        * The algorithm used for signing
    * @return signingAlgorithm
    **/
        @javax.annotation.Nullable
      @ApiModelProperty(value = "The algorithm used for signing")
    
    public String getSigningAlgorithm() {
        return signingAlgorithm;
    }


    public void setSigningAlgorithm(String signingAlgorithm) {
        this.signingAlgorithm = signingAlgorithm;
    }


        public WebsubSubscriptionConfigurationDTO signatureHeader(String signatureHeader) {
        
        this.signatureHeader = signatureHeader;
        return this;
        }

    /**
        * The header uses to send the signature
    * @return signatureHeader
    **/
        @javax.annotation.Nullable
      @ApiModelProperty(value = "The header uses to send the signature")
    
    public String getSignatureHeader() {
        return signatureHeader;
    }


    public void setSignatureHeader(String signatureHeader) {
        this.signatureHeader = signatureHeader;
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) {
        return true;
        }
        if (o == null || getClass() != o.getClass()) {
        return false;
        }
            WebsubSubscriptionConfigurationDTO websubSubscriptionConfiguration = (WebsubSubscriptionConfigurationDTO) o;
            return Objects.equals(this.enable, websubSubscriptionConfiguration.enable) &&
            Objects.equals(this.secret, websubSubscriptionConfiguration.secret) &&
            Objects.equals(this.signingAlgorithm, websubSubscriptionConfiguration.signingAlgorithm) &&
            Objects.equals(this.signatureHeader, websubSubscriptionConfiguration.signatureHeader);
    }

    @Override
    public int hashCode() {
        return Objects.hash(enable, secret, signingAlgorithm, signatureHeader);
    }


@Override
public String toString() {
StringBuilder sb = new StringBuilder();
sb.append("class WebsubSubscriptionConfigurationDTO {\n");
    sb.append("    enable: ").append(toIndentedString(enable)).append("\n");
    sb.append("    secret: ").append(toIndentedString(secret)).append("\n");
    sb.append("    signingAlgorithm: ").append(toIndentedString(signingAlgorithm)).append("\n");
    sb.append("    signatureHeader: ").append(toIndentedString(signatureHeader)).append("\n");
sb.append("}");
return sb.toString();
}

/**
* Convert the given object to string with each line indented by 4 spaces
* (except the first line).
*/
private String toIndentedString(Object o) {
if (o == null) {
return "null";
}
return o.toString().replace("\n", "\n    ");
}

}

